//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#include "graph_node.hpp"
#include "graph_visitor.hpp"
#include "utils.hpp"

using namespace std;
using namespace jdl;

//dag::dag_node::~dag_node(){
//	Logger::getInstance().debug().out << "DELETING DAG NODE: " << this->name << endl; 
//	std::vector<dag_node *>::iterator it = children.begin();
//	while(it != children.end())
//		delete *(it++);
//}

void graph::node::add_child(boost::shared_ptr<node> node){
	assert(node != NULL); 
	children.push_back(node);
	node->add_parent(this);
}

void graph::node::accept(graph::const_visitor &v) const{
	v.visit(*this);
}

void graph::node::accept(graph::visitor &v){
	v.visit(*this);
}

void graph::node::print(int indentation, ostream& out) const{
	indent(indentation, out);
	out << name << endl;
	vector<boost::shared_ptr<graph::node> >::const_iterator it = children.begin();
	while(it != children.end())
		(*it++)->print(indentation+1);
}

string build_path(vector<string> const& nodes){
	std::ostringstream path;
	path << "[";
	vector<string>::const_iterator it = nodes.begin();
	while(it != nodes.end()){
		path << *it;
		if(++it != nodes.end())
			path << ", ";
	}
	path << "]";
	return path.str();
}

void graph::acyclic_visitor::visit(graph::node const& node){
	if(find(nodes.begin(), nodes.end(), node.get_name()) != nodes.end()){
		string path = build_path(nodes);
		throw semantic_error("DAG is not acyclic, node " + node.get_name() + " already exists in path: " + path);
	}
	nodes.push_back(node.get_name());
	vector<boost::shared_ptr<graph::node> >::const_iterator it2 = node.get_children().begin();
	while(it2 != node.get_children().end())
		(*it2++)->accept(*this);
	assert (nodes.back() == node.get_name());
	nodes.pop_back();
}

void graph::dump_visitor::visit(graph::node const& node){
	indent(indentation, out);
	out << node.get_name() << endl;
	indentation++;
	vector<boost::shared_ptr<graph::node> >::const_iterator it = node.get_children().begin();
	while(it != node.get_children().end())
		(*it++)->accept(*this);
	indentation--;
}
