//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#ifndef DAG_TREE_HPP_
#define DAG_TREE_HPP_

#include "tree.hpp"

using namespace logger;
using namespace jdl;

namespace graph{
//////////////////////////////////////////////////////////////
// DAG TREE
//////////////////////////////////////////////////////////////
class visitor;
	
class node{
	std::string name;
	component const* definition;
	std::vector<node*> parents;
	std::vector<boost::shared_ptr<node> > children;
	
	void add_parent(node* node){ parents.push_back(node); }
		
public:
	node(std::string _name, jdl::component const* def): 
		name(_name), definition(def){ }
	  
	void add_child(boost::shared_ptr<node> node);
	
	std::string get_name() const { return name; }
	
	component const* get_definition() const { return definition; }
	
	std::vector<node*> const& get_parents() const { return parents; }
	
	std::vector<boost::shared_ptr<node> > const& get_children() const { return children; }
	
	void print(int indentation, ostream& out = cout) const;
	
	void accept(graph::const_visitor& v) const;
	
	void accept(graph::visitor& v);
	
	//~dag_node();
};

}

#endif /*DAG_TREE_HPP_*/
