//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#ifndef DAG_VISITOR_HPP_
#define DAG_VISITOR_HPP_

#include "graph_node.hpp"

namespace graph{
	
class const_visitor{
public:
	virtual void visit(node const& node) = 0;
	virtual ~const_visitor(){}
};

class visitor{
public:
	virtual void visit(node& node) = 0;
	virtual ~visitor(){}
};
	
class dump_visitor : public const_visitor{
	int 		indentation;
	ostream& 	out;	
public:
	dump_visitor(int indent_): indentation(indent_), out(cout){}
	
	dump_visitor(int indent_, ostream& out_): indentation(indent_), out(out_){}
	
	void visit(node const& node);
	
	~dump_visitor(){}
};
	
class acyclic_visitor : public const_visitor{
	std::vector<std::string> nodes;
public:
	void visit(node const& node);
	
	~acyclic_visitor(){}
};
	
}

#endif /*DAG_VISITOR_HPP_*/
