//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#include "jdl_parser.hpp"

using namespace jdl;

bool jdl::parse_string(jdl_parser& parser, std::string const& jdl){
	if(jdl.length() == 0)
		return false;
	try{
		return parser.parse(jdl);
	}catch(parser_error<Errors, char const *> ex){
		std::ostringstream err;
		err << "Syntax error occurred: " << endl;
		switch(ex.descriptor){
			case semicolon_expected:
				err << "';' expected";
				break;
			case right_quad_par_expected:
				err << "']' expected";
				break;
			case right_round_par_expected:
				err << "')' expected";
				break;
			case right_graph_par_expected:
				err << "'}' expected";
				break;
			case colon_expected:
				err << "':' expected";
				break;
			case attribute_expected: 
				err << "valid attribute expected";
				break;
			default:
				err << "Error occured";
		}
		pair<int,int> err_desc = iter2RowCol(jdl, ex.where);
		throw jdl::parse_error(err.str(), err_desc.first, err_desc.second);
	}catch(semantic_error e){
		string error = e.what();
		pair<int,int> err_desc = iter2RowCol(jdl, e.where);
		throw jdl::parse_error(error, err_desc.first, err_desc.second);
	}
}
