//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#ifndef JDL_PARSER_HPP_
#define JDL_PARSER_HPP_

#include "defs.hpp"
#include "jdl_grammar.hpp"

namespace jdl{

class jdl_parser{
	jdl_grammar grammar;
	parser_stack<boost::shared_ptr<component> > node_stack;
public:
	jdl_parser(): grammar(node_stack){
#ifdef DEBUG
		BOOST_SPIRIT_DEBUG_NODE(grammar);
#endif
	}
	
	bool parse(string const& str) 
		throw(parser_error<Errors, char const *>, semantic_error)
	{
		reset_parser();
		return boost::spirit::parse(str.c_str(),grammar,space_p).full;
	}
	
	component const* get_AST() const{ 
		if(node_stack.top())
			return node_stack.top().get();
		return NULL;
	} 
	
	void reset_parser(){ node_stack.clear(); }
	
	~jdl_parser(){ }
};

bool parse_string(jdl_parser& parser, std::string const& jdl);

inline bool parse_file(jdl_parser& parser, std::string const& file_name){
	return parse_string(parser, open_file(file_name));
}

}

#endif /*JDL_PARSER_HPP_*/
