//  This file is part of jdl_parser.
//
//  jdl_parser is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  jdl_parser is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with jdl_parser.  If not, see <http://www.gnu.org/licenses/>.

// Copyright Simone Pellegrini 2007-2008; e-mail: motonacciu@gmail.com

#ifndef VISITOR_HPP_
#define VISITOR_HPP_

#include "defs.hpp"
#include "tree.hpp"
#include "graph_node.hpp"

namespace jdl{

class const_visitor{
public:
	virtual void visit(int_node const& node) = 0;
	virtual void visit(float_node const& node) = 0;
	virtual void visit(string_node const& node) = 0;
	virtual void visit(string_literal_node const& node) = 0;
	virtual void visit(binary_expr_node const& node) = 0;
	virtual void visit(attribute_definition_node const& node) = 0;
	virtual void visit(conditional_expr const& node) = 0;
	virtual void visit(dot_node const& node) = 0;
	virtual void visit(bracket_node const& node) = 0;
	virtual void visit(term_node const& node) = 0;
	virtual void visit(list_node const& node) = 0;
	virtual void visit(func_call_node const& node) = 0;
	virtual void visit(unary_expr_node const& node) = 0;
	virtual ~const_visitor(){ };
};

class visitor{
public:
	virtual void visit(int_node &node) = 0;
	virtual void visit(float_node &node) = 0;
	virtual void visit(string_node &node) = 0;
	virtual void visit(string_literal_node &node) = 0;
	virtual void visit(binary_expr_node &node) = 0;
	virtual void visit(attribute_definition_node &node) = 0;
	virtual void visit(conditional_expr &node) = 0;
	virtual void visit(dot_node &node) = 0;
	virtual void visit(bracket_node &node) = 0;
	virtual void visit(term_node &node) = 0;
	virtual void visit(list_node &node) = 0;
	virtual void visit(func_call_node &node) = 0;
	virtual void visit(unary_expr_node &node) = 0;
	virtual ~visitor(){ };
};

//template <class Ret, class Arg>
//class arg_visitor{
//	public:
//		virtual Ret visit(int_node &node, Arg &arg) = 0;
//		virtual Ret visit(float_node &node, Arg &arg) = 0;
//		virtual Ret visit(string_node &node, Arg &arg) = 0;
//		virtual Ret visit(binary_expr_node &node, Arg &arg) = 0;
//		virtual Ret visit(attribute_definition_node &node, Arg &arg) = 0;
//		virtual Ret visit(conditional_expr &node, Arg &arg) = 0;
//		virtual Ret visit(dot_node &node, Arg &arg) = 0;
//		virtual Ret visit(bracket_node &node, Arg &arg) = 0;
//		virtual Ret visit(term_node &node, Arg &arg) = 0;
//		virtual Ret visit(func_call_node &node, Arg &arg) = 0;
//		virtual Ret visit(unary_expr_node &node, Arg &arg) = 0;
//		virtual ~arg_visitor(){}
//};

class base_const_visitor : public const_visitor{
void visit_node(const abstract_node *node);
public:
	void visit(int_node const& node){}
	void visit(string_node const& node){}
	void visit(string_literal_node const& node){};
	void visit(float_node const& node){}
	void visit(binary_expr_node const& node){ visit_node(&node); }
	void visit(attribute_definition_node const& node){ visit_node(&node); }
	void visit(conditional_expr const& node){ visit_node(&node); }
	void visit(dot_node const& node){ visit_node(&node); }
	void visit(bracket_node const& node){ visit_node(&node);}
	void visit(term_node const& node){ visit_node(&node); }
	void visit(list_node const& node){ visit_node(&node); }
	void visit(func_call_node const& node){ visit_node(&node); }
	void visit(unary_expr_node const& node){ visit_node(&node); }
};

class base_visitor : public visitor{
void visit_node(abstract_node *node);
public:
	void visit(int_node &node){}
	void visit(string_node &node){}
	void visit(string_literal_node &node){};
	void visit(float_node &node){}
	void visit(binary_expr_node &node){ visit_node(&node); }
	void visit(attribute_definition_node &node){ visit_node(&node); }
	void visit(conditional_expr &node){ visit_node(&node); }
	void visit(dot_node &node){ visit_node(&node); }
	void visit(bracket_node &node){ visit_node(&node);}
	void visit(term_node &node){ visit_node(&node); }
	void visit(list_node &node){ visit_node(&node); }
	void visit(func_call_node &node){ visit_node(&node); }
	void visit(unary_expr_node &node){ visit_node(&node); }
};

//class arg_base_visitor : public arg_visitor<string,string>{
//	string visit_node(abstract_node *node, string &arg){
//		
//	}
//	public:
//		string visit(int_node &node, string &arg){ return "" + node.data(); }
//		string visit(float_node &node, string &arg){ return node.data() + ""; }
//		string visit(string_node &node, string &arg){ return "" + node.data(); }
//		string visit(binary_expr_node &node, string &arg){ return arg;}
//		string visit(attribute_definition_node &node, string &arg){ return arg; }
//		string visit(conditional_expr &node, string &arg){ return arg; }
//		string visit(dot_node &node, string &arg){ return arg; }
//		string visit(bracket_node &node, string &arg){ return arg; }
//		string visit(term_node &node, string &arg){ return arg; }
//		string visit(func_call_node &node, string &arg){ return arg; }
//		string visit(unary_expr_node &node, string &arg){ return arg; }
//		~arg_base_visitor(){}
//};

class dump_visitor : public base_const_visitor{
	int indentation;
	std::ostream &out;
public:
	dump_visitor() : indentation(0), out(std::cout){}
	dump_visitor(ostream &o, int indent = 0) : indentation(indent), out(o){}
	void visit(int_node const& node);
	void visit(float_node const& node);
	void visit(string_node const& node);
	void visit(string_literal_node const& node);
	void visit(binary_expr_node const& node);
	void visit(attribute_definition_node const& node);
	void visit(conditional_expr const& node);
	void visit(dot_node const& node);
	void visit(bracket_node const& node);
	void visit(func_call_node const& node);
	void visit(term_node const& node);
	void visit(list_node const& node);
	void visit(unary_expr_node const& node);
};


class check_type_visitor: public base_const_visitor{
	string type;
	bool found;
public:
	check_type_visitor(): found(false){}
	string get_type() const { return type; }
	void visit(attribute_definition_node const& node);
};

class find_visitor: public base_const_visitor{
	vector<string> path;
	vector<int> selector;
	
	string const p;
	
	size_t depth;
	component const* value;
	attribute_definition_node const* attribute;
	
public:
	find_visitor(std::string const& _path);
	
	component const* get_value() const { return value; }
	attribute_definition_node const* get_attribute() const { return attribute; }
	vector<string> const& get_path() const { return path; }
	vector<int> const& get_selector() const { return selector; }
	
	void visit(attribute_definition_node const& node);
};

class dag_visitor : public base_const_visitor{
	map<string, component*> nodes;
	vector<pair<string,string> > dependencies;
	bool start_record;
	bool start_node;
	bool is_dag_node;
	bool start_dependecies;
	bool dep_list;
	bool found_nodes;
	bool found_deps;
	graph::node* root;
	
	void check_dag();
public:
	dag_visitor(): 
		start_record(false), 
		start_node(false), 
		is_dag_node(false),
		start_dependecies(false),
		dep_list(false),
		found_nodes(false),
		found_deps(false){}
	void visit(attribute_definition_node const& node);
	
	void visit(list_node const& node);
	
	graph::node const* get_DAG() const { return root; }
	
	~dag_visitor(){  delete root; }
};

class collection_visitor: public base_const_visitor{
	graph::node* root;
public:
	collection_visitor(){ root = new graph::node("ROOT", NULL); }
	graph::node const* get_DAG() const { return root; }
	
	void visit(attribute_definition_node const& node);
	
	~collection_visitor(){}
};

}

#endif /*VISITOR_HPP_*/
